@@ -2,12 +2,10 @@ use std::collections::HashMap;
 
 use tmelcrypt::{Ed25519PK, Ed25519SK};
 
-pub(in crate::testing) const GENESIS_MEL_SUPPLY: u128 = 21_000_000;
 pub(in crate::testing) const GENESIS_NUM_STAKERS: u64 = 10;
 pub(in crate::testing) const GENESIS_EPOCH_START: u64 = 0;
 pub(in crate::testing) const GENESIS_EPOCH_POST_END: u64 = 1000;
 pub(in crate::testing) const GENESIS_STAKER_WEIGHT: u128 = 100;
-pub(in crate::testing) const SEND_MEL_AMOUNT: u128 = 30_000_000_000;
 
 lazy_static! {
     pub static ref DB: novasmt::Database<novasmt::InMemoryCas> =
