@@ -1,14 +1,20 @@
 #[cfg(test)]
 pub mod bench;
 
+#[cfg(test)]
+pub mod constants;
+
+#[cfg(test)]
+pub mod factory;
+
 #[cfg(test)]
 pub mod fixtures;
 
 #[cfg(test)]
-pub mod utils;
+pub mod functions;
 
 #[cfg(test)]
 pub mod integration;
 
 #[cfg(test)]
-pub mod factory;
+pub mod utils;
\ No newline at end of file
